import { EventBus, IRule, RuleProps } from '@aws-cdk/aws-events';
import { FunctionProps, IFunction } from '@aws-cdk/aws-lambda';
import { IQueue, QueueProps } from '@aws-cdk/aws-sqs';
import { Construct, Duration } from '@aws-cdk/core';
export interface EventFunctionRuleProps extends RuleProps {
    eventBus: EventBus;
}
export interface EventFunctionDeliveryProps {
    /**
     * The maximum age of a request that Lambda sends to a function for processing.
     */
    maxEventAge?: Duration;
    /**
     * The maximum number of times to retry when the function returns an error.
     */
    retryAttempts?: number;
}
export interface EventFunctionProps {
    /**
     * The configuration and behaviour of the EventBridge Rule which links the EventBus and the Event Function Lambda
     */
    ruleProps: EventFunctionRuleProps;
    /**
     * Configuration of the Event Function Lambda
     */
    functionProps: FunctionProps;
    /**
     * Configuration of the Delivery properties to the Event Function Lambda
     */
    deliveryProps?: EventFunctionDeliveryProps;
    /**
     * Configuration of the Delivery DLQ
     */
    deliveryDlqProps?: QueueProps;
    /**
     * Configuration of the Processing DLQ
     */
    processingDlqProps?: QueueProps;
}
/**
 * EventFunction
 *
 * A Lambda Function, triggered by an EventBridge Event
 *
 * Includes additional resources:
 *   - Configurable EventBridge Rule
 */
export declare class EventFunction extends Construct {
    /**
     * Lambda Function which is the target of the Event
     */
    readonly eventFunction: IFunction;
    /**
     * EventBride Rule which controls when the Event Function is triggered
     */
    readonly rule: IRule;
    /**
     * The SQS Queue which acts as a DLQ for any EventBridge Delivery failures
     */
    readonly deliveryDlq: IQueue;
    /**
     * The SQS Queue which acts as a DLQ for any events which the Event Function is unable to process
     */
    readonly processingDlq: IQueue;
    constructor(scope: Construct, id: string, props: EventFunctionProps);
}
