import { EventBus, IRule, Rule, RuleProps } from '@aws-cdk/aws-events'
import { LambdaFunction } from '@aws-cdk/aws-events-targets'
import { Function, FunctionProps, IFunction } from '@aws-cdk/aws-lambda'
import { IQueue, Queue, QueueProps } from '@aws-cdk/aws-sqs'
import { Construct, Duration } from '@aws-cdk/core'

export interface EventFunctionRuleProps extends RuleProps {
  eventBus: EventBus
}

export interface EventFunctionDeliveryProps {
  /**
   * The maximum age of a request that Lambda sends to a function for processing.
   */
  maxEventAge?: Duration
  /**
   * The maximum number of times to retry when the function returns an error.
   */
  retryAttempts?: number
}

export interface EventFunctionProps {
  /**
   * The configuration and behaviour of the EventBridge Rule which links the EventBus and the Event Function Lambda
   */
  ruleProps: EventFunctionRuleProps
  /**
   * Configuration of the Event Function Lambda
   */
  functionProps: FunctionProps
  /**
   * Configuration of the Delivery properties to the Event Function Lambda
   */
  deliveryProps?: EventFunctionDeliveryProps
  /**
   * Configuration of the Delivery DLQ
   */
  deliveryDlqProps?: QueueProps
  /**
   * Configuration of the Processing DLQ
   */
  processingDlqProps?: QueueProps
}

/**
 * EventFunction
 *
 * A Lambda Function, triggered by an EventBridge Event
 *
 * Includes additional resources:
 *   - Configurable EventBridge Rule
 */
export class EventFunction extends Construct {
  /**
   * Lambda Function which is the target of the Event
   */
  readonly eventFunction!: IFunction
  /**
   * EventBride Rule which controls when the Event Function is triggered
   */
  readonly rule!: IRule
  /**
   * The SQS Queue which acts as a DLQ for any EventBridge Delivery failures
   */
  readonly deliveryDlq!: IQueue
  /**
   * The SQS Queue which acts as a DLQ for any events which the Event Function is unable to process
   */
  readonly processingDlq!: IQueue

  constructor(scope: Construct, id: string, props: EventFunctionProps) {
    const {
      ruleProps,
      functionProps,
      deliveryProps,
      deliveryDlqProps,
      processingDlqProps,
    } = props
    super(scope, id)

    const processingDlq = new Queue(
      this,
      `${id}-processing-dlq`,
      processingDlqProps
    )

    const eventFunction = new Function(this, `${id}-function`, {
      environment: {
        PROCESSING_DLQ_ARN: processingDlq.queueArn,
        PROCESSING_DLQ_NAME: processingDlq.queueName,
        ...functionProps.environment,
      },
      ...functionProps,
    })
    processingDlq.grantSendMessages(eventFunction)

    const rule = new Rule(this, `${id}-rule`, ruleProps)

    const deliveryDlq = new Queue(this, `${id}-delivery-dlq`, deliveryDlqProps)

    rule.addTarget(
      new LambdaFunction(eventFunction, {
        deadLetterQueue: deliveryDlq,
        ...deliveryProps,
      })
    )
  }
}
